### Persephone - a socio-economic-ecological model of European agricultural landscapes.
###
### These are the tests for the nature model (excluding individual species).
###

@testset "Species macros" begin
    #TODO
end

@testset "Habitat macros" begin
    # set up the testing landscape
    model = smalltestlandscape(Union{Animal,FarmPlot})
    model.landscape[6,6] = Pixel(Ps.agriculture, 1, [])
    species::Dict{String,Any} = Dict("name"=>"test_animal")
    add_agent!((6,6), FarmPlot, model, [(6,6)], Ps.wheat, 1.2, 3.4)
    add_agent!((3,3), Animal, model, species, Ps.male, 1)
    add_agent!((4,4), Animal, model, species, Ps.female, 1)
    # create a set of habitat descriptors
    h1 = @habitat(Ps.@landcover() == Ps.water)
    h2 = @habitat(Ps.@croptype() == Ps.wheat &&
                  Ps.@cropheight() < 2)
    h3 = @habitat(Ps.@distanceto(water) > 2 &&
                  Ps.@distancetoedge() <= 2)
    h4 = @habitat(Ps.@countanimals("test_animal", 1) == 1)
    # test the descriptors
    @test h1((6,4), model) == true
    @test h1((5,4), model) == false
    @test h2((6,6), model) == true
    @test h2((5,6), model) == false
    @test h3((3,3), model) == true
    @test h3((5,4), model) == false
    @test h3((6,1), model) == false
    @test h4((2,2), model) == true
    @test h4((3,3), model) == false
    @test h4((1,1), model) == false
end

@testset "Species initialisation" begin
    model = smalltestlandscape()
    spec = "test_animal"
    species::Dict{String,Any} = Dict("name"=>spec)
    # create a set of initialisation functions
    initfun1 = Ps.initrandompopulation(10)
    initfun2 = Ps.initrandompopulation(6*6*3, asexual=true)
    initfun3 = Ps.initpopulation(@habitat(Ps.@landcover() == Ps.grass), pairs=true)
    initfun4 = Ps.initpopulation(@habitat(Ps.@landcover() == Ps.water &&
                                          Ps.@countanimals("test_animal", 0) < 5),
                                 popsize=10)
    # apply and test the initialisation functions
    @test_logs (:info, "Initialised 10 $(spec)s.") initfun1(species, model)
    @test all(a -> a.sex in (Ps.male, Ps.female), allagents(model))
    genocide!(model)
    @test_logs (:info, "Initialised 108 $(spec)s.") initfun2(species, model)
    @test Ps.countanimals((1,1), model, spec, 0) == Ps.countanimals((6,6), model, spec, 0) == 3
    @test all(a -> a.sex == Ps.hermaphrodite, allagents(model))
    genocide!(model)
    @test_logs (:info, "Initialised 16 $(spec)s.") initfun3(species, model)
    @test Ps.countanimals((2,2), model, spec, 2) == Ps.countanimals((5,3), model, spec, 1) == 0
    @test Ps.countanimals((5,5), model, spec, 0) == Ps.countanimals((6,6), model, spec, 0) == 2
    a1, a2 = Ps.nearby_animals((6,6), model, 0)
    @test a1.sex != a2.sex
    genocide!(model)
    @test_logs((:warn, "There are not enough suitable locations for $(spec) in the landscape."),
               (:info, "Initialised 5 $(spec)s."),
               initfun4(species, model))
    @test Ps.countanimals((1,1), model, spec, 4) == 0
    @test Ps.countanimals((6,4), model, spec, 0) == 5
end

@testset "Population functions" begin
    #TODO
end
